body {
    font-family: "source sans pro", "clear sans", "liberation sans",
    "arial", "sans-serif";
    background: white;
}
div.title {
    text-align: center;
    font-weight: bold;
    margin: 0.67em 0;
}
h1 {
    font-size: 2em;
    margin: 0;
}
span.subtitle {
    font-size: 1.25em;
    font-style: italic;
}
code, pre {
    font-family: "source code pro", "liberation mono", "monospace";
}
pre, blockquote {
    margin-left: 4em;
    margin-right: 4em;
}
code {
    display: inline;
    white-space: nowrap;
}
a {
    text-decoration: none;
}
div.toc {
    padding-left: 0;
    font-size: 195%;
}
div.toc li {
    list-style-type: none;
    padding-left: 0;
}
div.toc ol {
    padding-left: 2em;
    font-size: 80%;
}
li.toc1 {
    padding-top: 0.7em;
}
li.toc2 {
    padding-top: 0.3em;
}
ul.index {
    list-style-type: none;
}
@media not screen {
    ul.navbar {
	display: none;
    }
}
@media print {
    a {
	color: inherit;
    }
}
@media only screen {
    div.contents {
	-webkit-column-gap: 4em;
	-webkit-column-rule: 1px dotted black;
	-moz-column-gap: 4em;
	-moz-column-rule: 1px dotted black;
	column-gap: 4em;
	column-rule: 1px dotted black;
    }
}
@media only screen and (min-width: 90em) {
    /* For a very wide screen, go to a columnar layout */
    div.contents {
	-webkit-column-count: 2;
	-moz-column-count: 2;
	column-count: 2;
    }
}
@media only screen and (min-width: 135em) {
    div.contents {
	-webkit-column-count: 3;
	-moz-column-count: 3;
	column-count: 3;
    }
}
@media screen {
    /* Setting an explicit margin to keep the navbar from moving */
    body {
	padding: 0;
	margin: 8px;
    }

    /* Link styles */
    a:link {
	color: #33c;
    }
    a:visited {
	color: #338;
    }
    a:hover {
	background: #ccc;
    }
    a:active {
	color: #f33;
	background: #ccc;
    }

    /* Trick to avoid the navbar hiding the the target of an # link */
    :target {
	margin-top: -10vh;
	padding-top: 10vh;
	background: #ffa;	/* Highlight the jump target */
	background-clip: content-box;
    }

    ul.navbar {
	display: block;
	position: sticky;
	top: 8px;
	width: 100%;
	margin: 0;
	padding: 0;
	overflow: hidden;
	white-space: nowrap;
	list-style-type: none;
	background: #336 url("nasmlogw.png") no-repeat right center;
	background-size: contain;
    }

    ul.navbar li {
	float: left;
    }
    ul.navbar li.last {
	border-right: none;
    }
    ul.navbar a {
	border-right: 1px solid #bbb;
	display: block;
	color: white;
	text-align: center;
	padding: 1em 1.5em;
	text-decoration: none;
    }
    ul.navbar a:hover {
	background-color: #448;
    }
}
